<?php

namespace Drupal\media_entity_twig\Plugin\media\Source;

use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Template\TwigEnvironment;
use Drupal\media\MediaSourceBase;
use Drupal\media\MediaInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Media source for Twig templates.
 *
 * @MediaSource(
 *   id = "media_entity_twig",
 *   label = @Translation("Twig Template"),
 *   description = @Translation("Provides a media entity source for rendering Twig templates."),
 *   allowed_field_types = {"twig_field"},
 *   default_thumbnail_filename = "generic.png"
 * )
 */
class MediaEntityTwig extends MediaSourceBase implements ContainerFactoryPluginInterface {

  /**
   * The Twig environment service.
   *
   * Used to render Twig templates.
   *
   * @var \Drupal\Core\Template\TwigEnvironment
   */
  protected TwigEnvironment|NULL $twig = NULL;

  /**
   * {@inheritdoc}
   *
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   *   The service container.
   * @param array<string,mixed> $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin ID for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   *
   * @return static
   *   Returns an instance of the plugin.
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): static {
    $plugin = parent::create($container, $configuration, $plugin_id, $plugin_definition);

    if ($plugin instanceof self) {
      $plugin->twig = $container->get('twig');
    }

    return $plugin;
  }

  /**
   * {@inheritdoc}
   *
   * @return array<string,\Drupal\Core\StringTranslation\TranslatableMarkup>
   *   An array of metadata attributes provided by the Twig media source.
   */
  public function getMetadataAttributes(): array {
    return [
      'rendered' => $this->t('Rendered Twig'),
    ];
  }

  /**
   * {@inheritdoc}
   *
   * @param \Drupal\media\MediaInterface $media
   *   The media entity.
   * @param string $attribute_name
   *   The name of the metadata attribute to retrieve.
   *
   * @return string|null
   *   The rendered Twig template as a string, or NULL if not applicable.
   */
  public function getMetadata(MediaInterface $media, $attribute_name) {
    $field_name = $this->configuration['source_field'];
    $value = $media->get($field_name)->value;
    $twig = is_string($value) ? $value : '';

    switch ($attribute_name) {
      case 'rendered':
        return $this->twig?->renderInline($twig) ?? '';
    }
    return NULL;
  }

  /**
   * {@inheritdoc}
   *
   * @param \Drupal\media\MediaInterface $media
   *   The media entity.
   *
   * @return mixed
   *   The value of the source field.
   */
  public function getSourceFieldValue(MediaInterface $media) {
    $field_name = $this->configuration['source_field'];
    return $media->get($field_name)->value;
  }

}
